﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;

public class BalancePlatform : MonoBehaviour {
    public static BalancePlatform Instance { get; private set; }

	public BuildController buildController;
	public List<BalanceModifier> balanceModifiers = new List<BalanceModifier> ();
    public Transform rotatingTransform;

	public float weight;
	public float currentBalance;
	public float rotateSpeed = 0.1f;

	public float maxBalance;



	public UnityEvent OnBalanceMaxReached;
	public UnityEvent OnWaveCollided;

	public Text currentBalanceText;
	public Text currentWeightText;

	// Use this for initialization
    void Awake() {
        if (Instance != null) {
            Debug.LogError("Instance already exists");
            Destroy(gameObject);
            return;
        }

        //EnemySpawnManager spawn = new GameObject("Enemy Spawner").AddComponent<EnemySpawnManager>();
        //spawn.enemyPrefab = Resources.Load<Enemy>("Enemy");
        //spawn.timeUntilNextSpawn = spawn.intervalsOfSpawns = 5;

        Instance = this;
    }

	void Start () {
		CalculateCityWeight ();
		buildController.OnCityChanged.AddListener (CalculateCityWeight);
		OnBalanceMaxReached.AddListener (FullBoatSink);
	}
	
	// Update is called once per frame
	void Update () {
		
		currentBalance += weight * rotateSpeed * Time.deltaTime;
        UpdateBalanceModifiers();

        rotatingTransform.eulerAngles = new Vector3(0, 0, -currentBalance * 0.35f);

		if (Mathf.Abs(currentBalance) >= maxBalance)
			OnBalanceMaxReached.Invoke ();

		if(currentBalanceText != null)
			currentBalanceText.text = currentBalance.ToString ();
		if(currentWeightText != null)
		currentWeightText.text = weight.ToString ();
	}

	public void UpdateBalanceModifiers()
	{
		for (int i = balanceModifiers.Count - 1; i >= 0; i--) {
            BalanceModifier mod = balanceModifiers[i];
            float change = mod.modifier * Time.deltaTime;
            if (Mathf.Abs(change) > Mathf.Abs(mod.sum)) {
                change = mod.sum;
            }

            balanceModifiers[i].sum -= change;
            currentBalance += change;

            if (Mathf.Abs(mod.sum) <= 0) {
                balanceModifiers.RemoveAt(i);
            }
		}
	}

	public void CalculateCityWeight()
	{
		int calculatedWeight = 0;
		int middleIndex = ((buildController.grid.GetLength (0) / 2)  );
		for (int i = 0; i < buildController.grid.GetLength (0); i++) {
			for (int j = 0; j < buildController.grid.GetLength (1); j++) {
				if (buildController.grid [i, j] != null && buildController.grid [i, j].blockobject != null) {
					if (buildController.grid [i, j].weight != 0)
						calculatedWeight += i - middleIndex;
				}
			}
		}
		weight = calculatedWeight;
	}

	public void WaveCollided(BalanceModifier balanceModifier)
	{
		balanceModifiers.Add (balanceModifier);
		OnWaveCollided.Invoke ();
	}



    public void EnemyCollided(BalanceModifier balanceModifier) {
        balanceModifiers.Add(balanceModifier);
    }

	public void FullBoatSink() {
		int fall;
		int degree;
		enabled = false;
		if (currentBalance > 0) {
			fall = 3;
			degree = -90;

		} else {
			fall = -3;
			degree = 90;
		}
		iTween.RotateTo (rotatingTransform.gameObject, new Vector3 (0, 0, degree), 3);
		iTween.MoveBy(rotatingTransform.gameObject, iTween.Hash("x", fall, "easeType", "easeInOutExpo", "delay", 1));

	}

}
