﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.SceneManagement;

public class BuildController : MonoBehaviour {
    public bool builtThisTurn = false;

    public AudioSource gameMusic;

	public Vector3 blockzeropos;
	public GameObject buildingbase;
	public GameObject[] prefabs; //0 is for signifier, 1 is for base block, others are for everything else

	public TileObject[] TileObjectExamples;
	//public float[] weights;
	//public buildblock[][] grid = new buildblock[11][5]();
	public buildblock[,] grid = new buildblock[11, 6];

    public int CenterIndex { get { return grid.GetLength(0) / 2; } }
    public Vector3 CenterPosition { get { return grid[BuildController.instance.CenterIndex, 0].blockobject.transform.position; } }

	public float gridsize = 1f;

	public static BuildController instance;

	public TileObject currentTypeToBuild;

	public SpriteRenderer[] RoofRenderers;

	public Sprite[] backgroundSprites;

	public UnityEvent OnCityChanged;

	public GameObject clickedTile = null;

	public UnityEvent OnClick;

	public Sprite ClockSprite;

	public Sprite RoofCannon;

	public Sprite BuildingCannon;

	public Sprite Redsignifier;
	
	public Sprite Greensignifier;

	//List<GameObject> signifierblocks =  new List<GameObject>();

	public class buildblock{
		//public Vector3 pos = new Vector3(0f,0f,0f);
		public int x = 0;
		public int y = 0;
		public int type = 0;
		public float weight = 1f;
		public GameObject blockobject; 

		public buildblock(int type, int x, int y){
			//this.pos = buildpos;
			this.x = x;
			this.y = y;
			this.type = type;
			//this.weight = BuildController.instance.weights[type];
			GameObject block = (GameObject)Instantiate(BuildController.instance.prefabs[type], BuildController.instance.blockzeropos , Quaternion.identity);
			block.transform.parent = BuildController.instance.buildingbase.transform;
			block.transform.localPosition = new Vector3((this.x - 5) * BuildController.instance.gridsize, (this.y - 0.5f) * BuildController.instance.gridsize, 0f);
			block.transform.localRotation = Quaternion.identity;
			blockobject = block;
			this.weight = block.GetComponent<TileObject>().weight;
			BuildController.instance.grid[x,y] = this;
		}
	}

    public void GameOverMusic() {
        gameMusic.clip = Resources.Load<AudioClip>("Audio/GameOver");
        gameMusic.loop = false;
        gameMusic.Play();
    }

	// Use this for initialization
	void Awake(){
		instance = this;
        Cannon.Projectiles = null;
	}

	void Start () {
		//instance = this;
		InitializeSignifiers ();
        grid[CenterIndex - 1, 0] = Build(1, CenterIndex - 1, 0);
//        grid[CenterIndex, 0] = Build(1, CenterIndex, 0);
        grid[CenterIndex + 1, 0] = Build(1, CenterIndex + 1, 0);
		for (int i = 0; i<grid.GetLength(1); i++) {
			grid[CenterIndex, i] = Build(1, CenterIndex, i);
			grid[CenterIndex, i].blockobject.tag = "Untagged";
			grid[CenterIndex, i].blockobject.GetComponent<SpriteRenderer> ().sprite = backgroundSprites [0];
		}
		grid[CenterIndex, 0].blockobject.GetComponent<SpriteRenderer> ().sprite = ClockSprite;
//		grid[CenterIndex, 1] = Build(1, CenterIndex, 1);
//		grid[CenterIndex, 2] = Build(1, CenterIndex, 1);
//		grid[CenterIndex, 3] = Build(1, CenterIndex, 1);
//		grid[CenterIndex, 4] = Build(1, CenterIndex, 1);
//		grid[CenterIndex, 5] = Build(1, CenterIndex, 1);

		
		//grid [(grid.GetLength(0)  / 2) + 2, 0] = Build(1, 7, 0, 1f);

		DeActivateSignifiers ();
		refreshLineBackgrounds ();
		refreshRoofs ();

		//ActivateSignifiers ();

	}

    private void LateUpdate() {
        builtThisTurn = false;
    }

	buildblock Build(int type, int x, int y){
		if (grid [x, y] != null) {
			DestroyImmediate (grid [x, y].blockobject);
		}
		return new buildblock (type, x, y);
	}

	void InitializeSignifiers(){
		for (int i = 0; i < grid.GetLength(0); i++) {
			for (int j = 0; j < grid.GetLength(1); j++) {
				grid [i,j] = new buildblock (0, i, j);
				//grid[i,j].blockobject.SetActive(false);
			}
		}
	}

	public void RefreshSignifiers(){
		for (int i = 0; i < grid.GetLength(0); i++) {
			for (int j = 0; j < grid.GetLength(1); j++) {
				if(grid[i,j].type == 0 && (j == 0 || grid[i,j - 1].type != 0)){
					if(ResourceManager.instance.buildPrePlanning && currentTypeToBuild != null && !ResourceManager.instance.haveEnoughGold(currentTypeToBuild.cost)){
						grid[i,j].blockobject.GetComponent<SpriteRenderer> ().sprite = Redsignifier;
					}
					else{
						//if(!(currentTypeToBuild.type == 5 && j != 0 )){
							grid[i,j].blockobject.GetComponent<SpriteRenderer> ().sprite = Greensignifier;
						//}
						
					}
					//							grid[i,j].blockobject.GetComponent<SpriteRenderer> ().sprite = Greensignifier;

				}
			}
		}
	}

	public void DeActivateSignifiers(){
		for (int i = 0; i < grid.GetLength(0); i++) {
			for (int j = 0; j < grid.GetLength(1); j++) {
				if(grid[i,j].type == 0){
					grid[i,j].blockobject.SetActive(false);
				}
				else{
					//Debug.Log("" + i +" , " + j);
				}
			}
		}
	}

	public void ActivateSignifiers(){
		for (int i = 0; i < grid.GetLength(0); i++) {
			for (int j = 0; j < grid.GetLength(1); j++) {
				if(grid[i,j].type == 0 && (j == 0 || grid[i,j - 1].type != 0) && !(currentTypeToBuild != null && currentTypeToBuild.type == 5 && j!= 0) && HasAdjecentBuilt(i)){
					grid[i,j].blockobject.SetActive(true);
				}
			}
		}
		RefreshSignifiers ();
	}

	bool HasAdjecentBuilt(int i){
//		return (i== 0 && grid[i + 1,0].type != 0) ||
//			(i == grid.GetLength(1)-1 && grid[grid.GetLength(1)-2,0].type != 0) ||
//				(grid[i + 1,0].type != 0 || grid[i - 1,0].type != 0);
		if (i == 0){
			if( grid [i + 1, 0].type != 0) {
				return true;
			}
		} else if ((i == grid.GetLength (0) - 1 )){
			if(( grid [grid.GetLength (0) - 2, 0].type != 0)) {
				return true;
			}
		} else if ((grid [i + 1, 0].type != 0 || grid [i - 1, 0].type != 0)) {
			return true;
		}
		return false;
	}

	public void BuildTile(GameObject signifier){
		DeActivateSignifiers ();
		if (ResourceManager.instance.haveEnoughGold (currentTypeToBuild.cost)) {
			for (int i = 0; i < grid.GetLength(0); i++) {
				for (int j = 0; j < grid.GetLength(1); j++) {
					if (grid [i, j].blockobject == signifier) {
						grid [i, j] = Build (currentTypeToBuild.type, i, j);
					}
				}
			}
			ResourceManager.instance.substractGold (currentTypeToBuild.cost);

			OnCityChanged.Invoke ();
			refreshLineBackgrounds ();
			refreshRoofs ();
            SoundManager.PlaySound("Build");
            builtThisTurn = true;
		}
	}

	void refreshLineBackgrounds(){
//		for (int i=0; i< grid.GetLength(0); i++) {
//			ReplaceLineVisual(i);
//		}
		for (int i = 0; i < grid.GetLength(0); i++) {
			for (int j = 0; j < grid.GetLength(1); j++) {
				if(i != CenterIndex){
					DecideTileSprite(i,j);
				}
			}
		}
	}

	void refreshRoofs(){
		//		for (int i=0; i< grid.GetLength(0); i++) {
		//			ReplaceLineVisual(i);
		//		}
		for (int i = 0; i < grid.GetLength(0); i++) {
			int top = GetTopInLine(i);
			if(top == -1){
				RoofRenderers[i].gameObject.SetActive(false);
			}else{
				RoofRenderers[i].gameObject.SetActive(true);
				RoofRenderers[i].gameObject.transform.localPosition 
					= new Vector3((i - 5) * BuildController.instance.gridsize, (top + 0.5f) * BuildController.instance.gridsize, 0f);
			}
		}
	}

	int GetTopInLine(int i){ //returns the height of the highest block or -1 if it's a cannon or no blocks here
		int res = -1;
		for (int j = 0; j < grid.GetLength(1); j++) {
			if(i != CenterIndex){
				if(grid[i,j].type == 1){
					res = j;
				}
			}
		}
		return res;
	}

	public void SetCurrentBuildType(int type){
		currentTypeToBuild = prefabs[type].GetComponent<TileObject>();
		if(ResourceManager.instance.haveEnoughGold(currentTypeToBuild.cost)){
			DeActivateSignifiers();
			ActivateSignifiers();
		}
	}

//	void ReplaceLineVisual(int line){
//		int sum = 0;
//			for (int j = 0; j < grid.GetLength(1); j++) {
//			if(grid[line,j].type != 0){
//				sum++;
//			}
//		}
//		LineRenderers [line].sprite = backgroundSprites [sum];
//	}

	void DecideTileSprite(int x, int y){
		if (grid [x, y].type == 1) {
			if (y == 0 || grid [x, y - 1].type != 1 || (y > 3 && (grid [x, y - 1].type == 1 && grid [x, y - 2].type == 1 && grid [x, y - 3].type == 1) && grid [x, y - 4].type == 1)) {//bottom level or bottom of chain
				grid [x, y].blockobject.GetComponent<SpriteRenderer> ().sprite = backgroundSprites [Mathf.Min (4, tilesInColumn (x, y))];
			} else {
				grid [x, y].blockobject.GetComponent<SpriteRenderer> ().sprite = backgroundSprites [0];
			}
		} else if (grid [x, y].type == 2) {
				if (y == grid.GetLength(1) - 1 || grid [x, y+1].type == 0){
					grid [x, y].blockobject.GetComponent<SpriteRenderer> ().sprite = RoofCannon;
				}else{
					grid [x, y].blockobject.GetComponent<SpriteRenderer> ().sprite = BuildingCannon;
			}
			if(x > CenterIndex){
				grid [x, y].blockobject.GetComponent<SpriteRenderer> ().flipX = true;
			}
		} else if (grid [x, y].type == 3) {
			if(x > CenterIndex){
				grid [x, y].blockobject.GetComponent<SpriteRenderer> ().flipX = true;
			}
		}
	}

	int tilesInColumn(int x, int start){
		int sum = 0;
		for (int i = start; i < grid.GetLength(1); i++) {
			if(grid[x,i].type == 1){
				sum++;
			}
			else{
				break;
			}
		}
		return sum;
	}

	public bool CheckIfCanSell(GameObject tile){
		//public void BuildTile(GameObject signifier){
		//DeActivateSignifiers ();
		for (int i = 0; i < grid.GetLength(0); i++) {
			for (int j = 0; j < grid.GetLength(1); j++) {
				if(grid[i,j].blockobject == tile){
					if(j == grid.GetLength(1) - 1){
						//Debug.Log("Returning true for top");
						return true;
					}else{
						//Debug.Log("Returning " + grid[i,j+1].type == 0 + "for ok");
						return grid[i,j+1].type == 0;
					}
				}
			}
		}
		return false;
	}

	public void SellTile(GameObject tile){
		//public void BuildTile(GameObject signifier){
		//DeActivateSignifiers ();
		Debug.Log("searching for sell target");

		for (int i = 0; i < grid.GetLength(0); i++) {
			for (int j = 0; j < grid.GetLength(1); j++) {
				if(grid[i,j].blockobject == tile){
					Debug.Log("Selling " + i +" , " + j);
					ResourceManager.instance.addGold(grid[i,j].blockobject.GetComponent<TileObject>().cost/2);
					grid[i,j] =  Build(0, i, j);
				}
			}
		}
		DeActivateSignifiers ();
		refreshLineBackgrounds ();
		refreshRoofs ();
        SoundManager.PlaySound("Sell");
	}

	public void ActivateSewer(GameObject tile,float power,  float time ){
		//public void BuildTile(GameObject signifier){
		//DeActivateSignifiers ();
		//Debug.Log("searching for sell target");
		
		for (int i = 0; i < grid.GetLength(0); i++) {
			for (int j = 0; j < grid.GetLength(1); j++) {
				if(grid[i,j].blockobject == tile){
					BalancePlatform.Instance.WaveCollided (new BalanceModifier((-Mathf.Sign(i - CenterIndex))* power, time));
				}
			}
		}
		DeActivateSignifiers ();
		refreshLineBackgrounds ();
		refreshRoofs ();
	}

//	void InitializePool(){
//		for (int i = 0; i < 10; i++) {
//			GameObject block = (GameObject)Instantiate(BuildController.instance.prefabs[1], Vector3.zero , Quaternion.identity);
//			signifierblocks.Add(block);
//			block.SetActive(false);
//		}
//	}
//
//	GameObject getSingnifierblock(){
//		GameObject block;
//		for (int i = 0; i < signifierblocks.Count; i++) {
//			block = signifierblocks[i];
//			if(block.activeInHierarchy){
//				signifierblocks.Remove(block);
//				block.SetActive(true);
//				return block;
//			}
//		}
//		block = (GameObject)Instantiate(BuildController.instance.prefabs[1], Vector3.zero , Quaternion.identity);
//		return block;
//	}
//
//	void GiveSignifierblockBack(GameObject block){
//		block.SetActive (false);
//		signifierblocks.Add (block);
//	}
	
	// Update is called once per frame
	void Update () {
		bool hitflag = false;
		if (Input.GetMouseButtonDown(0) && !builtThisTurn) {
			Vector2 mousePos = Camera.main.ScreenToWorldPoint(Input.mousePosition);

			foreach(Collider2D col in Physics2D.OverlapPointAll(mousePos)){
				if(col.gameObject.CompareTag("Tile")){
				    //Debug.Log("Hit a Tile");

				    hitflag = true;
				    if(col.gameObject.Equals(clickedTile)){
					    DeActivateSignifiers();
					    col.gameObject.SendMessage("CloseMenu");
					    clickedTile = null;
				    } else {
					    if(clickedTile != null){
						    clickedTile.SendMessage("CloseMenu");
					    }

					    col.gameObject.SendMessage("OpenMenu");
					    clickedTile = col.gameObject;
				    }
				}
			}
			if(!hitflag && clickedTile != null){
				DeActivateSignifiers();
				clickedTile.SendMessage("CloseMenu");
				clickedTile = null;
				}			
		}

        if (Input.GetKeyDown(KeyCode.Escape)) {
            Quit();
        }
	}

    private void Quit() {
        SceneManager.LoadScene("Main Menu");
    }
}
