﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class EnemySpawnManager : MonoBehaviour {

	public EnemyBundle[] enemyBundles;

    public float randomInterval = 6;
    public float directedInterval = 3;
    public float intervalRandomizer = 1;
    public float timeBeforeStartSpawn = 10;

    public float minMultiplier = 0.5f;
    public float maxMultiplier = 3f;
    public float timeToMax = 600;

    private float nextRandom;
    private float nextDirected;
    private float currentMultiplier;
    private float timeSinceStart = 0;

    public Transform[] spawnLocations;

	IEnumerator Start () {
        nextRandom = float.MaxValue;
        nextDirected = float.MaxValue;
        currentMultiplier = minMultiplier;

        while (timeBeforeStartSpawn > 0) {
            yield return null;
            timeBeforeStartSpawn -= Time.deltaTime;
        }

        nextRandom = GetNextSpawn(randomInterval);
        nextDirected = GetNextSpawn(directedInterval);
	}

    private float GetNextSpawn(float interval) {
        float result = interval + Random.Range(-intervalRandomizer, intervalRandomizer);
        result /= currentMultiplier;
        //Debug.Log("Next spawn: " + result);
        return result;
    }

	void Update()
	{
        nextRandom -= Time.deltaTime;
        nextDirected -= Time.deltaTime;
        timeSinceStart += Time.deltaTime;
        currentMultiplier = Mathf.Lerp(minMultiplier, maxMultiplier, timeSinceStart / timeToMax);

        if (nextRandom <= 0) {
            SpawnRandom();
            nextRandom = GetNextSpawn(randomInterval);
        }

        if (nextDirected <= 0) {
            SpawnDirected();
            nextDirected = GetNextSpawn(directedInterval);
        }
	}

    private void SpawnRandom() {
        float spawnX = spawnLocations[Random.Range(0, spawnLocations.Length)].position.x;
        Spawn(spawnX);
    }

    private void SpawnDirected() {
        float spawnX = BalancePlatform.Instance.currentBalance > 0 ? spawnLocations[0].position.x : spawnLocations[1].position.x;
        Spawn(spawnX);
    }

	//public 
	public void Spawn(float spawnX)
	{
        EnemyBundle bundle = enemyBundles[Random.Range(0, enemyBundles.Length)];

        for (int i = 0; i < bundle.count; i++) {
            Enemy enemy = bundle.enemies[Random.Range(0, bundle.enemies.Length)];
            float spawnY = Random.Range(enemy.minY, enemy.maxY);

            Enemy instance = GameObject.Instantiate(enemy, new Vector3(spawnX, spawnY), Quaternion.identity);
            if (spawnX < 0f) {
                instance.GetComponent<SpriteRenderer>().flipX = true;
            }
        }
	}
}
