﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class WaveSpawnManager : MonoBehaviour {

	public GameObject wave;
    public WaveBundle[] series;

	public float intervalWithinBundle = 2;
	public float intervalBetweenBundles = 10;

    public Transform[] spawnLocations;

    private Queue<WaveBundle> nextBundles;
    private WaveBundle currentBundle;
    private int currentWave;
    private float nextWaveTime;
    private float nextBundleTime;
    private Transform currentWaveSpawn;

    private void Start() {
        nextBundles = new Queue<WaveBundle>(series);
        nextBundleTime = intervalBetweenBundles;
    }

    private void Update() {
        nextBundleTime -= Time.deltaTime;
        nextWaveTime -= Time.deltaTime;

        if (nextBundleTime <= 0) {
            StartNextBundle();
        }

        if (nextWaveTime <= 0 && currentBundle != null) {
            SpawnWave();
        }
    }

    private void StartNextBundle() {
        nextBundleTime = int.MaxValue;
        currentBundle = nextBundles.Dequeue();
        nextBundles.Enqueue(currentBundle);
        currentWave = 0;
        currentWaveSpawn = spawnLocations[Random.Range(0, spawnLocations.Length)];
        SpawnWave();
    }

	public void SpawnWave()
	{
		GameObject.Instantiate(wave, currentWaveSpawn.position, Quaternion.identity);
        nextWaveTime = intervalWithinBundle;
        currentWave++;

        if (currentWave >= currentBundle.count) {
            currentWave = 0;
            nextBundleTime = intervalBetweenBundles;
            currentBundle = null;
        }
	}
}
